@@ -71,6 +71,23 @@ pub struct ClusterConnection {
     password: Option<String>,
     read_timeout: RefCell<Option<Duration>>,
     write_timeout: RefCell<Option<Duration>>,
+    tls: Option<TlsMode>,
+}
+
+#[derive(Clone, Copy)]
+enum TlsMode {
+    Secure,
+    Insecure,
+}
+
+impl TlsMode {
+    fn from_insecure_flag(insecure: bool) -> TlsMode {
+        if insecure {
+            TlsMode::Insecure
+        } else {
+            TlsMode::Secure
+        }
+    }
 }
 
 impl ClusterConnection {
@@ -83,7 +100,6 @@ impl ClusterConnection {
             Self::create_initial_connections(&initial_nodes, readonly, password.clone())?;
 
         let connection = ClusterConnection {
-            initial_nodes,
             connections: RefCell::new(connections),
             slots: RefCell::new(SlotMap::new()),
             auto_reconnect: RefCell::new(true),
@@ -91,6 +107,26 @@ impl ClusterConnection {
             password,
             read_timeout: RefCell::new(None),
             write_timeout: RefCell::new(None),
+            #[cfg(feature = "tls")]
+            tls: {
+                if initial_nodes.is_empty() {
+                    None
+                } else {
+                    // TODO: Maybe should run through whole list and make sure they're all matching?
+                    match &initial_nodes.get(0).unwrap().addr {
+                        ConnectionAddr::Tcp(_, _) => None,
+                        ConnectionAddr::TcpTls {
+                            host: _,
+                            port: _,
+                            insecure,
+                        } => Some(TlsMode::from_insecure_flag(*insecure)),
+                        _ => None,
+                    }
+                }
+            },
+            #[cfg(not(feature = "tls"))]
+            tls: None,
+            initial_nodes,
         };
         connection.refresh_slots()?;
 
@@ -166,6 +202,14 @@ impl ClusterConnection {
         for info in initial_nodes.iter() {
             let addr = match info.addr {
                 ConnectionAddr::Tcp(ref host, port) => format!("redis://{}:{}", host, port),
+                ConnectionAddr::TcpTls {
+                    ref host,
+                    port,
+                    insecure,
+                } => {
+                    let tls_mode = TlsMode::from_insecure_flag(insecure);
+                    build_connection_string(host, Some(port), Some(tls_mode))
+                }
                 _ => panic!("No reach."),
             };
 
@@ -180,7 +224,7 @@ impl ClusterConnection {
         if connections.is_empty() {
             return Err(RedisError::from((
                 ErrorKind::IoError,
-                "It is failed to check startup nodes.",
+                "It failed to check startup nodes.",
             )));
         }
         Ok(connections)
@@ -246,7 +290,7 @@ impl ClusterConnection {
         let mut samples = connections.values_mut().choose_multiple(&mut rng, len);
 
         for mut conn in samples.iter_mut() {
-            if let Ok(mut slots_data) = get_slots(&mut conn) {
+            if let Ok(mut slots_data) = get_slots(&mut conn, self.tls) {
                 slots_data.sort_by_key(|s| s.start());
                 let last_slot = slots_data.iter().try_fold(0, |prev_end, slot_data| {
                     if prev_end != slot_data.start() {
@@ -395,7 +439,9 @@ impl ClusterConnection {
                         let kind = err.kind();
 
                         if kind == ErrorKind::Ask {
-                            redirected = err.redirect_node().map(|x| format!("redis://{}", x.0));
+                            redirected = err
+                                .redirect_node()
+                                .map(|(node, _slot)| build_connection_string(node, None, self.tls));
                             is_asking = true;
                         } else if kind == ErrorKind::Moved {
                             // Refresh slots.
@@ -403,7 +449,9 @@ impl ClusterConnection {
                             excludes.clear();
 
                             // Request again.
-                            redirected = err.redirect_node().map(|x| format!("redis://{}", x.0));
+                            redirected = err
+                                .redirect_node()
+                                .map(|(node, _slot)| build_connection_string(node, None, self.tls));
                             is_asking = false;
                             continue;
                         } else if kind == ErrorKind::TryAgain || kind == ErrorKind::ClusterDown {
@@ -448,7 +496,7 @@ impl ClusterConnection {
         let mut results = vec![Value::Nil; cmds.len()];
 
         let to_retry = self
-            .send_all_commands(&cmds)
+            .send_all_commands(cmds)
             .and_then(|node_cmds| self.recv_all_commands(&mut results, &node_cmds))?;
 
         if to_retry.is_empty() {
@@ -505,7 +553,7 @@ impl ClusterConnection {
         let mut cmd_map: HashMap<String, NodeCmd> = HashMap::new();
 
         for (idx, cmd) in cmds.iter().enumerate() {
-            let addr = self.get_addr_for_cmd(&cmd)?;
+            let addr = self.get_addr_for_cmd(cmd)?;
             let nc = cmd_map
                 .entry(addr.clone())
                 .or_insert_with(|| NodeCmd::new(addr));
@@ -682,7 +730,7 @@ fn get_random_connection<'a>(
 }
 
 // Get slot data from connection.
-fn get_slots(connection: &mut Connection) -> RedisResult<Vec<Slot>> {
+fn get_slots(connection: &mut Connection, tls_mode: Option<TlsMode>) -> RedisResult<Vec<Slot>> {
     let mut cmd = Cmd::new();
     cmd.arg("CLUSTER").arg("SLOTS");
     let value = connection.req_command(&cmd)?;
@@ -728,11 +776,11 @@ fn get_slots(connection: &mut Connection) -> RedisResult<Vec<Slot>> {
                         }
 
                         let port = if let Value::Int(port) = node[1] {
-                            port
+                            port as u16
                         } else {
                             return None;
                         };
-                        Some(format!("redis://{}:{}", ip, port))
+                        Some(build_connection_string(&ip, Some(port), tls_mode))
                     } else {
                         None
                     }
@@ -750,3 +798,17 @@ fn get_slots(connection: &mut Connection) -> RedisResult<Vec<Slot>> {
 
     Ok(result)
 }
+
+fn build_connection_string(host: &str, port: Option<u16>, tls_mode: Option<TlsMode>) -> String {
+    let host_port = match port {
+        Some(port) => format!("{}:{}", host, port),
+        None => host.to_string(),
+    };
+    match tls_mode {
+        None => format!("redis://{}", host_port),
+        Some(TlsMode::Insecure) => {
+            format!("rediss://{}/#insecure", host_port)
+        }
+        Some(TlsMode::Secure) => format!("rediss://{}", host_port),
+    }
+}
